import requests
from .request import Request


class CfHttp:
    __ua = 'Mozilla/5.0'

    def __init__(self, cf_proxy: str, ua: str = Request.user_agent):
        self.cf_proxy = cf_proxy
        self.__ua = ua

    def get(self, url, **kwargs) -> requests.Response:
        return self.__req('get', '')

    def __req(self, method, url, **kwargs) -> requests.Response:
        headers = kwargs.get('headers', {})
        kwargs.update({
            'cmd': f'request.{method}',
            'url': url,
            'userAgent': headers.get('User-Agent', self.__ua),
            'maxTimeout': int(kwargs.get('maxTimeout', 60000)),
        })
        return requests.post(f'{self.cf_proxy}/v1', **kwargs)



from manga_py.provider import Provider
from .helpers.std import Std


class MangaOwlNet(Provider, Std):
    def get_archive_name(self) -> str:
        return self.chapter[1]

    def get_chapter_index(self) -> str:
        return self.chapter[1]

    def get_content(self):
        return self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return self.text_content(self.content, '.single_detail h2')

    def get_chapters(self):
        chapters = self.document_fromstring('a.chapter-url', self.content)
        return [
            (c.get('data-href'), c.cssselect('label.chapter-title'), ) for c in chapters
        ]

    def get_files(self):
        parser = self.document_fromstring(self.http_get(self.chapter[0]))
        return self._images_helper(parser, 'img.ows-lazy', 'data-src', 'src')

    def get_cover(self) -> str:
        return self._cover_from_content('.single_detail a>img', 'data-src')

    def chapter_for_json(self) -> str:
        return self.chapter[0]


main = MangaOwlNet



requirements = ["cloudscraper","cssselect>=1.1","lxml~=4.6","packaging>=20.3","Pillow~=8.1","progressbar2>=3.50","pycryptodome>=3.9","PyExecJS>=1.5","requests>=2.23","better_exceptions>=0.2","pysocks>=1.7","toml>=0.10"]


from typing import Optional
from requests import post, Response
from . import Http as Http_
from base64 import b64decode


class Http:
    sid: str = None

    def __init__(self, solver_url: str, http: Http_):
        self.headers = {'Content-Type': 'application/json'}
        self.solver_url = solver_url
        self.user_agent = http.user_agent
        self.cookies = {}

    def create_session(self, sid: str = None) -> Optional[str]:
        self.sid = post(self.solver_url, json={
            'cmd': 'sessions.create',
            'userAgent': self.user_agent,
            'session': sid,
        }).json().get('session', None)
        return sid

    def destroy_session(self):
        post(self.solver_url, json={
            'session': self.sid,
        })

    def get(self, url, headers: dict = None, cookies: dict = None, **kwargs) -> Response:
        cookies = [{'name': k, 'value': cookies[k]} for k in cookies] if cookies is not None else []
        return post(self.solver_url, json={
            'session': self.sid,
            'cmd': 'request.get',
            'url': url,
            'userAgent': self.user_agent,
            'headers': headers or {},
            'cookies': cookies,
            **kwargs,
        }, headers=self.headers)

    def post(self, url, headers: dict = None, data: dict = None, cookies: dict = None, **kwargs) -> Response:
        cookies = [{'name': k, 'value': cookies[k]} for k in cookies] if cookies is not None else []
        post_data = data or {}
        encoded_data = []
        for k in post_data:
            encoded_data.append(f'{k}={post_data[k]}')
        headers = headers or {}
        headers.setdefault('Content-Type', 'application/x-www-form-urlencoded')
        return post(self.solver_url, json={
            'session': self.sid,
            'cmd': 'request.post',
            'url': url,
            'userAgent': self.user_agent,
            'headers': headers,
            'postData': encoded_data,
            'cookies': cookies,
        }, headers=self.headers)

    def file(self, url, headers: dict = None, cookies: dict = None, **kwargs) -> bytes:
        response = self.get(url, headers, download=True, cookies=cookies)
        return b64decode(response.json().get('solution', {}).get('response'))



from manga_py.provider import Provider
from .helpers.std import Std


class CatMangaOrg(Provider, Std):
    _build_id = None

    @property
    def _content(self) -> dict:
        return self.content.get('props', {}).get('pageProps', {}).get('series', {})

    def get_archive_name(self) -> str:
        return self.normal_arc_name({
            'vol': str(self.chapter.get('volume')),
            'ch': str(self.chapter.get('number'))},
        )

    def get_chapter_index(self) -> str:
        return self.chapter_for_json()

    def get_content(self):
        url = self.re.search(r'(https?://.+?/series/[^/]+)', self.get_url()).group(1)
        data = self.json.loads(
            self.re.search(
                r'>(\{"(?:query|props|page|buildId|isFallback|gsp)"\:\{.+?\})<',
                self.http_get(url),
                self.re.MULTILINE).group(1),
        )
        return data

    def _url_manga_name(self):
        return self._get_name(r'/series/([^/]+)')

    def get_manga_name(self) -> str:
        titles = self._content.get('alt_titles', [])
        if len(titles) > 0:
            return titles[0]
        return self._url_manga_name()

    def get_chapters(self):
        return self._content.get('chapters', [])[::-1]

    def get_files(self):
        # https://catmanga.org/_next/data/<buildId>/series/<name>/<number>.json
        data = self.json.loads(self.http_get(
            '{}/_next/data/{}/series/{}/{}.json'.format(
                self.domain,
                self.content.get('buildId'),
                self.content.get('query', {}).get('series') or self._url_manga_name(),
                self.chapter.get('number', 0)
            )
        ))
        return data.get('pageProps', {}).get('pages')

    def get_cover(self) -> str:
        return self._content.get('cover_art', {}).get('source', '')

    def chapter_for_json(self) -> str:
        number = self.chapter.get('number')
        volume = self.chapter.get('volume')

        if volume is not None:
            return f'{number}-{volume}'
        return str(number)


main = CatMangaOrg



from manga_py.provider import Provider
from .helpers.std import Std


class GenkanIo(Provider, Std):
    def get_chapter_index(self) -> str:
        return self.chapter[1]

    def get_content(self):
        return self.http_get(self.re.search(
            r'(https?://.+?/manga/[^/]+)',
            self.get_url()
        ).group(1))

    def get_manga_name(self) -> str:
        parser = self.document_fromstring(self.content, 'h2', 0)
        return self.element_text_content_full(parser)

    def get_chapters(self):
        return [(el.get('href'), self.element_text_content_full(el)) for el in self._elements('section table td:nth-child(1) > a')]

    def get_files(self):
        parser = self.html_fromstring(self.chapter[0])
        return self._images_helper(parser, '.container > img')

    def get_cover(self) -> str:
        return self._cover_from_content('section > div > img')

    def prepare_cookies(self):
        self.http().referer = self.domain
        self.http().allow_send_referer = True

    def chapter_for_json(self) -> str:
        return self.chapter[0]


main = GenkanIo



from manga_py.provider import Provider
from .helpers.std import Std


class FireCrossJp(Provider, Std):
    def get_chapter_index(self) -> str:
        pass

    def get_archive_name(self) -> str:
        pass

    def get_content(self):
        self.http_get(self.get_url())

    def get_manga_name(self) -> str:
        return self.text_content(self.content, '.ebook-series-title')

    def get_chapters(self) -> list:
        elements = self._elements(
            'li.shop-item--episode'
        )
        return [(
            x.get('data-id'),
            x.cssselect('.shop-item-info-name')[0].text.strip,
        ) for x in elements][::-1]

    def get_files(self) -> list:
        # scramble
        """
        <Scramble>0,9,3,12,8,6,13,1,11,2,15,5,14,4,7,10</Scramble>
        curl
         'https://firecross.jp/celsys/diazepam_hybrid.php?mode=8&file=0000.xml&reqtype=0&vm=4&param=1T6XoHEliHqVjZKvm%2FobiMljUv7qhn0Dv%2FVWs8eky6CduoTZYSjUWMmaxE4Kcf2GxFWkkyiFaDvRS3r6Mnd6U3%2Bh1d5IzmxcKX2rGPsU%2F2BuZyQfoazlp%2BUWX09AbIb6eo8RqoPSv7am128jP%2Bd0jg%3D%3D&time=379934'
         -H 'User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:97.0) Gecko/20100101 Firefox/97.0'
         -H 'Accept: */*' -H 'Accept-Language: ru-RU,ru;q=0.8,en-US;q=0.5,en;q=0.3'
         -H 'Accept-Encoding: gzip, deflate, br'
         -H 'Referer: https://firecross.jp/reader/3349?trial=0&token=0be2513c-8cc7-434d-8afd-e9bc389cfa9e'
         -H 'DNT: 1'
         -H 'Connection: keep-alive'
         -H 'Cookie: XSRF-TOKEN=%3D; _s=%3D'
         -H 'Sec-Fetch-Dest: empty'
         -H 'Sec-Fetch-Mode: cors'
         -H 'Sec-Fetch-Site: same-origin'
         -H 'Sec-GPC: 1'
         -H 'TE: trailers'
        :return:
        """

        # total pages
        """
        <TotalPage>26</TotalPage>
        https://firecross.jp/celsys/diazepam_hybrid.php?mode=7&file=face.xml&reqtype=0&vm=4&param=1T6XoHEliHqVjZKvm%2FobiMljUv7qhn0Dv%2FVWs8eky6CduoTZYSjUWMmaxE4Kcf2GxFWkkyiFaDvRS3r6Mnd6U3%2Bh1d5IzmxcKX2rGPsU%2F2BuZyQfoazlp%2BUWX09AbIb6eo8RqoPSv7am128jP%2Bd0jg%3D%3D&time=379934
        """

        # image
        """
        https://firecross.jp/celsys/diazepam_hybrid.php?mode=1&file=0001_0000.bin&reqtype=0&vm=4&param=1T6XoHEliHqVjZKvm%2FobiMljUv7qhn0Dv%2FVWs8eky6CduoTZYSjUWMmaxE4Kcf2GxFWkkyiFaDvRS3r6Mnd6U3%2Bh1d5IzmxcKX2rGPsU%2F2BuZyQfoazlp%2BUWX09AbIb6eo8RqoPSv7am128jP%2Bd0jg%3D%3D&time=379934
        """

        content_html = self.http_get(self.chapter[0])

        reader = self.document_fromstring(content_html, '#meta', 0)

        cgi_value = reader.cssselect('input[name="cgi"]')
        param_value = reader.cssselect('input[name="param"]')

        mode_value = 8
        file_value = '%04d.xml'
        reqtype_value = '0'
        vm_value = 4
        time_value = '6889374'  # ?

        return []

    def _get_chapter_url(self):
        """
        curl 'https://firecross.jp/api/reader'
            -X POST
            -H 'User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:97.0) Gecko/20100101 Firefox/97.0'
            -H 'Accept: application/json'
            -H 'Accept-Language: ru-RU,ru;q=0.8,en-US;q=0.5,en;q=0.3'
            -H 'Accept-Encoding: gzip, deflate, br'
            -H 'Content-Type: application/x-www-form-urlencoded; charset=UTF-8'
            -H 'X-Requested-With: XMLHttpRequest'
            -H 'Origin: https://firecross.jp'
            -H 'DNT: 1'
            -H 'Connection: keep-alive'
            -H 'Referer: https://firecross.jp/ebook/series/440'
            -H 'Cookie: XSRF-TOKEN=%3D; _s=%3D'
            -H 'Sec-Fetch-Dest: empty'
            -H 'Sec-Fetch-Mode: cors'
            -H 'Sec-Fetch-Site: same-origin'
            -H 'Sec-GPC: 1'
            --data-raw '_token=MQbdBgUC1bo3CQDL3g3iJ5sPGrxjjgJ0NGlRwXH3&ebook_id=3349'
        :return:
        """

        url = self.http().post(
            url=f'{self.domain}/api/reader',
            headers={
                'X-Requested-With': 'XMLHttpRequest',
                'Accept': 'application/json',
                'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            }
        )

        return url.json()['redirect']


main = FireCrossJp



from pathlib import Path


CONFIG_FILE = Path.home().joinpath('manga-py.toml')


def _available_keys():
    return [
        'auto_skip_deleted', 'cbz', 'create_empty_files', 'destination', 'global_progress',
        'max_threads', 'min_free_space', 'no_progress', 'no_webp', 'not_change_files_extension',
        'override_archive_name', 'proxy', 'rename_pages', 'rewrite_exists_archives',
        'skip_incomplete_chapters', 'user_agent', 'wait_after_chapter', 'wait_after_page', 'zero_fill'
    ]


def dump_init_content():
    if CONFIG_FILE.is_file():
        return

    data = """# default manga-py config. You can change the values as you like. See the "manga-py --help" for the meaning of the keys.
[default]
#auto_skip_deleted=
#cbz=
#create_empty_files=
#destination=
#global_progress=
#max_threads=
#min_free_space=
#no_progress=
#no_webp=
#not_change_files_extension=
#override_archive_name=
#proxy=
#rename_pages=
#rewrite_exists_archives=
#skip_incomplete_chapters=
#user_agent=
#wait_after_chapter=
#wait_after_page=
#zero_fill=
"""
    with CONFIG_FILE.open('w') as w:
        w.write(data)


class DefaultConfig:
    params = None

    def __init__(self, params: dict = None):
        self.params = {k: params[k] for k in params if k in _available_keys()}

    def __getattr__(self, item):
        return (self.params or {}).get(item, None)

    def get_all(self) -> dict:
        return (self.params or {}).copy()



from toml import load, dump
from . import DefaultConfig, CONFIG_FILE


def load_config() -> DefaultConfig:
    if not CONFIG_FILE.is_file():
        return DefaultConfig()

    with CONFIG_FILE.open('r') as r:
        data = load(r)
        return DefaultConfig(data.get('default', {}))


def dump_config(config: DefaultConfig):
    with CONFIG_FILE.open('r') as r:
        data = load(r)

    data['default'] = config.get_all()

    with CONFIG_FILE.open('w') as w:
        dump(data, w)



import argparse
import platform

import pkg_resources
from ..meta import version
from ._requirements import requirements

requires = []


def package_version(name: str) -> str:
    try:
        pkg = __import__(name)
        return pkg.__version__
    except ImportError:
        try:
            return pkg_resources.get_distribution(name).version
        except pkg_resources.DistributionNotFound:
            return ''


def print_versions(versions):
    length = [max(len(v[i]) for v in versions) for i in [0, 1]]

    for line in versions:
        print(f"{line[0]:{length[0]}} | {line[1]:{length[1]}}")


class DebugVersionAction(argparse.Action):
    def __init__(
            self,
            option_strings,
            dest,
            default=None,
            required=False,
            help=None,
    ):
        super().__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=0,
            default=default,
            required=required,
            help=help
        )

    def print_versions_help(self):
        versions = [
            ("Component", "Version"),
            ("---", "---"),
            ("OS", platform.platform(aliased=True)),
            ("python", platform.python_version()),
            ("pip", package_version('pip')),
            ("manga-py", version),
        ]
        print_versions(versions)
        print("")
        module_versions = [
            ("Module", "Version"),
            ("---", "---"),
        ]
        module_versions.extend(
            sorted(
                [(req, package_version(req)) for req in requirements]
            )
        )
        print_versions(module_versions)

    def __call__(self, parser, namespace, values, option_string=None):
        self.print_versions_help()
        exit(0)


def _args_debug(args_parser):  # pragma: no cover
    args = args_parser.add_argument_group('Debug / Simulation options')

    args.add_argument(
        '-h',
        '--help',
        action='help',
        help=(
            'Show this help and exit.'
        )
    )

    args.add_argument(
        '--print-json',
        action='store_true',
        help=(
            'Print information about the results in the JSON format (after completion).'
        )
    )

    args.add_argument(
        '--simulate',
        action='store_true',
        help=(
            'Simulate running %(prog)s, where: '
            '1) do not download files and, '
            '2) do not write anything on disk.'
        )
    )

    # deprecated
    args.add_argument(
        '--show-current-chapter-info',
        action='store_true',
        help=(
            'Show current processing chapter info.'
        )
    )

    # deprecated
    args.add_argument(
        '--save-current-chapter-info',
        action='store_true',
        help=(
            'Save current processing chapter info into a JSON file.'
        )
    )

    args.add_argument(
        '--show-chapter-info',
        action='store_true',
        help=(
            'Show current processing chapter info.'
        )
    )

    args.add_argument(
        '--save-chapter-info',
        action='store_true',
        help=(
            'Save current processing chapter info into a JSON file.'
        )
    )

    args.add_argument(
        '--save-manga-info',
        action='store_true',
        help=(
            'Saves the manga data in a into a JSON file at the root of the download folder.'
        )
    )

    args.add_argument(
        '--debug',
        action='store_true',
        help=(
            'Debug %(prog)s.'
        )
    )

    args.add_argument(
        '--debug-version',
        action=DebugVersionAction,
        help=(
            'Print debug version info.'
        )
    )

    args.add_argument(
        '-q',
        '--quiet',
        action='store_true',
        help=(
            'Dont show any messages.'
        )
    )



from .rawdevart_com_old import RawDevArtComOld


class LilyMangaCom(RawDevArtComOld):
    def get_chapter_index(self) -> str:
        return self.re.search(r'/(?:episode|chapter)-(\d+(?:-\d+)?)', self.chapter).group(1)

    def get_content(self):
        return self._get_content('{domain}/ys/{manga_name}/', manga_name=self._name())

    def get_manga_name(self) -> str:
        return self._name().replace('-', ' ')

    def get_chapters(self):
        manga_id = self._elements('#manga-chapters-holder')[0].get('data-id')
        items_content = self.http().post('{}/wp-admin/admin-ajax.php'.format(self.domain), data={
            'action': 'manga_get_chapters',
            'manga': manga_id,
        }).text

        return self._elements('.wp-manga-chapter > a', items_content)

    def _name(self):
        return self._get_name('/ys/([^/]+)')


main = LilyMangaCom



from manga_py.provider import Provider
from .helpers.std import Std


class ComicNaverCom(Provider, Std):
    title_id = 0
    url_pattern = 'https://comic.naver.com/webtoon/list.nhn?titleId={}'

    def get_archive_name(self) -> str:
        return self.get_chapter_index()

    def get_chapter_index(self) -> str:
        return 'vol_{}-{}'.format(self.chapter_id, self.chapter[1])

    def get_content(self):
        return self.http_get(self.url_pattern.format(self.title_id))

    def get_manga_name(self) -> str:
        return self.text_content(self.content, '.comicinfo > .detail > h2')

    def _chapters_part(self, content: str):
        elements = self._elements('.viewList .title > a', content)
        pager = self._elements('.page > .num_page', content)

        if len(pager) == 0:
            return elements, 1

        return elements, int(self.element_text_content_full(pager[-1]))

    def get_chapters(self):
        self.log('Parse chapters, please wait...')
        chapters, max_page = self._chapters_part(self.content)
        page = 2

        while True:
            if page >= max_page:
                break

            content = self.http_get(self.url_pattern.format(f'{self.title_id}&page={page}'))
            _chapters, max_page = self._chapters_part(content)

            chapters = [*chapters, *_chapters]

            page += 1

        n = self.http().normalize_uri
        return [(n(chapter.get('href')), self.element_text_content(chapter)) for chapter in chapters]

    def get_files(self):
        parser = self.document_fromstring(self.http_get(self.chapter[0]), '#comic_view_area', 0)
        return self._images_helper(parser, '.wt_viewer > img')

    def get_cover(self) -> str:
        return self._cover_from_content('.comicinfo > .thumb > a > img')

    def chapter_for_json(self) -> str:
        return self.chapter[0]

    def prepare_cookies(self):
        self.title_id = self.re.search(r'\btitleId=(\d+)', self.get_url()).group(1)


main = ComicNaverCom
